;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMB1 Firebar, by imamelia
;;
;; This is a Firebar that looks and acts like the one in SMB1.
;;
;; Uses first extra bit: YES
;; Uses extra property bytes: YES
;;
;; If the first extra bit is clear, the sprite will go clockwise, and if the first extra
;; bit is set, the sprite will go counterclockwise.  The extra property byte 1
;; determines how many fireballs, minus 1, should make up the whole bar; 05
;; would be 6 fireballs, like the normal-sized Firebar in SMB1, and 0B would be 12
;; fireballs, like the giant Firebar in SMB1.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;incsrc subroutinedefsx.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!RAM_Offsets = $7F9CE0	; RAM for the offsets of each individual fireball
; This should be RAM in some unused area.  You'll need 2 bytes for each fireball.
; For example, for the 6-tile Firebar, this should be at least 12 (0xC) bytes long.
; For the 12-tile Firebar, this should be at least 24 (0x18) bytes long.

!Speed = $03

Tilemap:
db $2C,$2D,$2C,$2D

Flip:
db $00,$00,$C0,$C0

Radius:
db $00,$08,$10,$18,$20,$28,$30,$38,$40,$48,$50,$58

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
PHB
PHK
PLB
JSR SpriteInit
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteInit:

LDA #!Speed
STA $7FAD00,x
LDA $7FAB10,x
AND #$04
BEQ NoInvertSpeed
LDA $7FAD00,x
EOR #$FF
INC
STA $7FAD00,x
NoInvertSpeed:
LDA #$00
STA $7FAC54,x
STA $7FAC60,x

LDA $D8,x
CLC
ADC #$04
STA $D8,x
STA $7FAD24,x
LDA $14D4,x
ADC #$00
STA $7FAD3C,x
LDA $E4,x
CLC
ADC #$04
STA $7FAD30,x
LDA $14E0,x
ADC #$00
STA $7FAD48,x

JSR MainEllipseCode

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR SpriteMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpriteMain:

JSR MainEllipseCode
JSR FirebarGFX

LDA $14C8,x
CMP #$08
BNE Return00
LDA $9D
BNE Return00

JSR SubOffscreenX0
;JSL !SubOffscreenX0

Return00:
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FirebarGFX:

JSR GetDrawInfo

LDA $14
LSR #2
AND #$03
TAX
LDA Flip,x
STX $04
LDX $15E9
ORA $15F6,x
ORA $64
STA $02
LDX $04
LDA Tilemap,x
STA $03

LDX $15E9
LDA $7FAD30,x
SEC
SBC $1A
STA $00
LDA $7FAD24,x
SEC
SBC $1C
STA $01
LDA $7FAB28,x
ASL
TAX

GFXLoop:

LDA $00
CLC
ADC !RAM_Offsets,x
STA $0300,y

LDA $01
CLC
ADC !RAM_Offsets+1,x
STA $0301,y

LDA $03
STA $0302,y

LDA $02
STA $0303,y

INY #4
DEX #2
BPL GFXLoop

LDX $15E9
LDY #$00
LDA $7FAB28,x
JSL $81B7B3

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; elliptical/circular motion subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MainEllipseCode:

JSR EllipseUpdateAngle	;
;JSL !EllipseUpdateAngle	;
JSR EllipseSetUpAngle	;
;JSL !EllipseSetUpAngle	;
JSR EllipseGetCoords	;
;JSL !EllipseGetCoords	;
JSR MultiplyAllCoords	;
;JSL !MultiplyAllCoords	;
JSR Interact		;

RTS

EllipseUpdateAngle:

LDA $9D
BNE Label92

LDY #$00
LDA $7FAD00,x
;CMP #$00
BPL Label91
DEY
Label91:
CLC
ADC $7FAD54,x
STA $7FAD54,x
TYA
ADC $7FAD60,x
AND #$01
STA $7FAD60,x

Label92:
RTS

EllipseSetUpAngle:

LDA $7FAD54,x
STA $00
LDA $7FAD60,x
STA $01
RTS

EllipseGetCoords:

REP #$30
LDA $00
CLC
ADC.w #$0080
AND #$01FF
STA $02
LDA $00
AND.w #$00FF
ASL A
TAX
LDA $07F7DB,x
STA $04
LDA $02
AND.w #$00FF
ASL A
TAX
LDA $07F7DB,x
STA $06
SEP #$30
RTS

MultiplyAllCoords:

LDX $15E9
LDA $04
STA $4202
LDA $7FAB28,x
ASL
TAX
STZ $0B
LDA $01
STA $0C

XCLoop:

LDA $0C
STA $01
LDA $0B
LDY $05
BNE Label93
STA $4203
NOP #4
ASL $4216
LDA $4217
ADC #$00
Label93:
LSR $01
BCC Label94
EOR #$FF
INC A
Label94:
STA !RAM_Offsets,x

LDA $0B
CLC
ADC #$08
STA $0B
DEX #2
BPL XCLoop

LDX $15E9
LDA $06
STA $4202
LDA $7FAB28,x
ASL
TAX
STZ $0B
LDA $03
STA $0C

YCLoop:

LDA $0C
STA $03
LDA $0B
LDY $07
BNE Label95
STA $4203
NOP #4
ASL $4216
LDA $4217
ADC #$00
Label95:
LSR $03
BCC Label96
EOR #$FF
INC A
Label96:
STA !RAM_Offsets+1,x

LDA $0B
CLC
ADC #$08
STA $0B
DEX #2
BPL YCLoop

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; interaction subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Interact:

LDX $15E9	;
LDA $7FAD24,x	; Y-coordinate of the circle's center, low byte
STA $0C		;
LDA $7FAD30,x	; X-coordinate of the circle's center, low byte
STA $0E		;
LDA $7FAD3C,x	; Y-coordinate of the circle's center, high byte
STA $0D		;
LDA $7FAD48,x	; X-coordinate of the circle's center, high byte
STA $0F		;

TXA		;
EOR $13		;
AND #$01	;
ORA $15A0,x	;
BEQ ProcessInteract	;
NoContact:	;
RTS		;

ProcessInteract:	;

LDA $71		;
BNE NoContact	;
LDA #$00		;
BIT $0D9B	;
BVS SkipCheckP	;
LDA $13F9,x	;
EOR $1632,x	;
SkipCheckP:	;
BNE NoContact2	;

JSL $83B664		; get player clipping routine
LDA #$06			;
STA $06			; sprite clipping width
STA $07			; sprite clipping height
LDA $7FAB28,x		;
ASL			;
TAX			;
ClippingLoop:		;
JSR SetSpriteClipping	; use custom sprite clipping values here
JSR CheckForContact	;
BCC EndClippingLoop	;
LDA $1490		; unless the player is invincible...
ORA $1497		;
BNE EndClippingLoop	;
JSL $80F5B7		; hurt him/her
EndClippingLoop:		;
DEX #2			;
BPL ClippingLoop		;
NoContact2:		;
LDX $15E9		;
RTS			;

SetSpriteClipping:		; custom sprite clipping routine, based off $03B69F

LDA !RAM_Offsets,x		; X displacement for a particular fireball
CLC			;
ADC $0E			;
CLC			;
ADC #$01		;
STA $04			; $04 = sprite X position low byte + X displacement value
LDA $0F			;
ADC #$00		;
STA $0A			; $0A = sprite X position high byte + X displacement high byte (00 or FF)

LDA !RAM_Offsets+1,x	; Y displacement for a particular fireball
CLC			;
ADC $0C			;
CLC			;
ADC #$01		;
STA $05			; $05 = sprite Y position low byte + Y displacement value
LDA $0D			;
ADC #$00
STA $0B			; $0B = sprite Y position high byte + Y displacement high byte (00 or FF)
RTS			;

CheckForContact:	;

PHX		;
LDX #$01		;

ContactLoop:	;

LDA $04,x	;
SEC		;
SBC $00,x		;
CLC		;
ADC $06,x	;
STA $185E	; I can't use $0F here because I'm already using it
LDA $02,x	;
CLC		;
ADC $06,x	;
CMP $185E	;
BCC EndCLoop	;
DEX		;
BPL ContactLoop	;

EndCLoop:	;
PLX		;
RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; miscellaneous standard subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

